// file      : xsd-frontend/traversal/enumeration.hxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2006 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_FRONTEND_TRAVERSAL_ENUMERATION_HXX
#define XSD_FRONTEND_TRAVERSAL_ENUMERATION_HXX

#include <xsd-frontend/traversal/elements.hxx>
#include <xsd-frontend/semantic-graph/enumeration.hxx>

namespace XSDFrontend
{
  namespace Traversal
  {
    struct Enumeration : ScopeTemplate<SemanticGraph::Enumeration>
    {
      virtual Void
      traverse (Type&);

      virtual Void
      pre (Type&);

      virtual Void
      name (Type&);

      template<typename X>
      Void
      inherits (Type& c,
                EdgeDispatcherBase& d,
		Void (X::*pre_) (Type&) = (Void (Enumeration::*)(Type&)) (0),
		Void (X::*post_) (Type&) = (Void (Enumeration::*)(Type&)) (0),
		Void (X::*none_) (Type&) = (Void (Enumeration::*)(Type&)) (0),
                Void (X::*next_) (Type&) = (Void (Enumeration::*)(Type&)) (0))
      {
        X* this_ (dynamic_cast<X*> (this));

        Type::InheritsIterator b (c.inherits_begin ()), e (c.inherits_end ());

        if (b != e)
        {
          if (pre_)
            (this_->*pre_) (c);

          iterate_and_dispatch (b, e, d, *this_, next_, c);

          if (post_)
            (this_->*post_) (c);
        }
        else
        {
          if (none_)
            (this_->*none_) (c);
        }
      }

      Void
      inherits (Type& c, EdgeDispatcherBase& d)
      {
        inherits<Enumeration> (c, d);
      }

      virtual Void
      inherits (Type&);

      virtual Void
      inherits_pre (Type&);

      virtual Void
      inherits_next (Type&);

      virtual Void
      inherits_post (Type&);

      virtual Void
      inherits_none (Type&);

      virtual Void
      post (Type&);
    };

    struct Enumerator : Node<SemanticGraph::Enumerator>
    {
      virtual Void
      traverse (Type&);

      virtual Void
      pre (Type&);

      virtual Void
      belongs (Type&, EdgeDispatcherBase&);

      virtual Void
      belongs (Type&);

      virtual Void
      name (Type&);

      virtual Void
      post (Type&);
    };
  }
}

#endif  // XSD_FRONTEND_TRAVERSAL_ENUMERATION_HXX
